; flash ROM board programmer

; with Arda's tape load error check routine

org 0x7800

start:
	di			; interrupts off
	jr	past_table
	
table:
	defb	$1b	; 9
	defb	$13	; 8
	defb	$0b	; 7
	defb	$03	; 6
	defb	$04	; 5
	defb	$0c	; 4
	defb	$14	; 3
	defb	$1c	; 2
	defb	$24	; 1
	defb	$23	; 0

past_table:
	
initial_paging:
	xor	a		; set a to zero
	ld	(X1FFD),a	; set the variables
	ld	(X7FFD),a	; set the variables
	ld	bc, 0x1ffd	; normal paging, high bit to 0
	out	(c), a
	ld	b, 0x7f		; RAM 0, ROM 0, screen 0
	out	(c), a

clear_screen:
	out	(0xfe), a	; border 0

	ld	hl, 0x4000	; clear bitmap
	ld	(hl), a
	ld	de, 0x4001
	ld	bc, 0x17ff
	ldir

	ld	hl, 0x5800	; attributes to
	ld	(hl), 0x47	; bright white on black
	ld	de, 0x5801
	ld	bc, 0x2ff
	ldir

set_stack:
	ld	hl, 0x7800	; stack to 0x77ff
	ld	sp, hl

display_menu:
	ld	hl, 0x1820	; PRINT AT 0,0
	ld	(s_posn), hl
	ld	hl, 0x4000	; screen area
	ld	(df_cc), hl
	xor	a		; a = 0
	ld	de, menu	; location of menu text
	call	po_msg		; print menu text

wait_key:
	push	hl
	call	key1		; wait for keypress
	pop	hl
	sub 0xf3	; reduce range
	push af
	ld bc, 0x5000
keyboard_pause:
	dec bc
	ld a,b
	or c
	jr nz, keyboard_pause	; busy wait
	pop af
	cp	0x0a		; number key 0 to 8?
	jr	nc, wait_key	; rescan if not
	
	push af
	ld	a, 0		
	out	(0xfe), a	; set a black border to clear any previous warning
	pop af
	call	decode
	jr	wait_key	
	
; subroutines
	
decode:	
	cp	0
	jp	z, copy_from_rom_bank_0
	cp	1
	jp	z, copy_from_rom_bank_1
	cp	2
	jp	z, copy_from_rom_bank_2
	cp	3
	jp	z, copy_from_rom_bank_3
	cp	4
	jp	z, change_ram_bank
	cp	5
	jp	z, change_rom_bank
	cp	6
	jp	z, load_rom_bank_from_tape
	cp	7
	jp	z, wipe_rom_bank
	cp	8
	jr	z, flash_rom_bank

boot_rom_bank:
	rst 0



flash_rom_bank:
	ld	hl, 0xc000	; source = 0xc000
	ld	bc, 0x4000	; 16K to write
	ld	de, 0x0000	; destination = 0x0000
	ld	a, (X1FFD)	; last write to port 0x1ffd
	bit	2, a		; test high ROM bit
	jr	z, do_flash	; jump if 0
	ld	d, 0x80		; destination = 0x8000
	jr 	do_flash	; jump

wipe_rom_bank:
	ld	de, 0x0000	; deFcastination = 0x0000
	ld	a, (X1FFD)	; last write to port 0x1ffd
	bit	2, a		; test high ROM bit
	jr	z, do_wipe	; jump if 0
	ld	d, 0x80		; destination = 0x8000
	jr 	do_wipe		; jump


do_flash:
	; the first job is to erase the sector. the sector we are going to flash
	; is already in DE so I can just call "do_wipe" here.
	; the sector that we want to flash data to will be wiped
	call	do_wipe
	
	; now that the sector is (hopefully) wiped we can write the data
	; do a loop writing the data to the flash until we have written all 16k

	
flash_a_byte:			; send the next byte to the flash chip
	push bc
	ld	bc, 0x0555	; unlock 1
	ld	a, 0xaa
	ld	(bc),a
	ld	bc, 0x02aa	; unlock 2
	ld	a, 0x55
	ld	(bc), a
	ld	bc, 0x0555	; set up
	ld	a, 0xa0
	ld	(bc), a
	ld	a, (hl)		; flash the byte
	ld	(de), a

fwb_wait:			; wait for byte write
	ld	b,d
	res	7,d
	ld	a,(de)		; read the status or data
	ld	d,b
	xor	(hl)		; which bits are ~data?
	bit	7,a			; .7: wait or error
	jr	z,fwb_test	; !.7: is it the correct whole byte?
fwb_eflag:			; test DQ5
	xor	(hl)		; status byte
	bit	5,a			; .5: error
	jr	z,fwb_wait	; !.5: wait

	;flash reset
	ld	a, 0xf0
	ld	(de), a
	ld	a, 2		; at this point something has clearly not worked
	out	(0xfe), a	; set a red border as a warning
	pop	bc
	ret				; drop back to the menu

fwb_test:			; check whole byte is correct
	and a			; a is (de)^(hl), should be 0 if (de)==(hl)
	jr z,fwb_exit	; if it was 0, we're done
	jr fwb_eflag	; see if it was a DQ5 error (else wait)

	
fwb_exit:
	pop bc	
	
	ldi
	
	ld a,b
	or c
	jr nz, flash_a_byte	; all 16k done? if not, loop
	ld 	a, 0x04
	out	(0xfe), a ; set border to green
	ret

; de holds the address to wipe
; send the programming codes
	
do_wipe:	
	push bc
	ld	bc, 0x0555	; unlock 1
	ld	a, 0xaa
	ld	(bc), a
	ld	bc, 0x02aa	; unlock 2
	ld	a, 0x55
	ld	(bc), a
	ld	bc, 0x0555	; set up
	ld	a, 0x80
	ld	(bc), a

; DUPLICATION BELOW... WHY?
	; +++the unlock codes have to be sent again before the erase command. they could be reused I suppose,+++
	; +++as long as you don't do an read or write operations which would reset the flash rom command circuitry+++

	ld	bc, 0x0555	; unlock 1
	ld	a, 0xaa
	ld	(bc), a
	ld	bc, 0x02aa	; unlock 2
	ld	a, 0x55
	ld	(bc), a
	
	ld	a, 0x30		; erase sector
	ld	(de), a

; routine partially nicked from Paul Farrow`s zxc3 flash programming routines
; www.fruitcake.plus.com
	
fse_wait:
	ld	b,d
	res	7,d
	ld	a,(de)		; read the status or data
	ld	d,b
	bit	7,a		; !.7: wait or error
	jr	nz, fse_test	; .7: finished
	bit	5,a		; .5: error
	jr	z,fse_wait	; !.5: wait
	; error: fall through
flash_reset:
	ld	a, 0xf0
	ld	(de), a
	ld	a, 2		; at this point something has clearly not worked
	out	(0xfe), a	; set a red border as a warning
	pop bc
	ret

fse_test:
	inc	a		; should have been 0xff so now 0
	jr	nz,flash_reset	; if nonzero, we have an error!
	; no error: fall through into fse_exit
fse_exit:
; end of flash routines
	ld a,0x06
	out (0xfe),a
	pop bc
	ret

change_rom_bank:
	ld	a, (X7FFD)	; last value written to 0x7ffd
	bit	4, a		; test ROM low bit
	jr	nz, inc_rom_h	; jump if 1

inc_rom_l:
	set	4, a		; increment ROM using only low bit
	ld	(X7FFD), a
	ld	bc, 0x7ffd
	out	(c), a
	ld	a, (X1FFD)
	ld	b, 0x1f
	out	(c), a
	jr	print_rom	

; low bit was 1

inc_rom_h:
	res	4, a		; clear ROM low bit
	ld	(X7FFD), a
	ld	bc, 0x7ffd
	out	(c), a	
	ld	a, (X1FFD)	; last value written to 0x1ffd 
	bit	2, a		; test ROM high bit
	jr	nz, res_h	; jump if 1
	set	2, a		; make it 1
	jr	rom_h_done

res_h:
	res	2, a		; make it 0

rom_h_done:
	ld	(X1FFD), a
	ld	bc, 0x1ffd
	out	(c), a

print_rom:
	ld	a, (X7FFD)	; get low bit
	and	00010000b	; mask off bit 4
	rra			; to bit 3
	rra			; to bit 2
	rra			; to bit 1
	rra			; to bit 0
	ld	b, a		; store in b
	ld	a, (X1FFD)	; get high bit
	and	00000100b	; mask off bit 2
	rra			; to bit 1
	add	a, b		; combine to form ROM number
	add	a, 48		; convert to ASCII
	ld	hl, 0x0b07	; PRINT AT 26,11
	ld	(s_posn), hl
	ld	hl, 0x48da
	ld	(df_cc), hl
	jp	print_out	; update menu

;---

change_ram_bank:
	ld	a, (X7FFD)	; get current value
	and	00000111b	; mask of RAM page
	cp	0		; is it 0
	jr	z, ram_1
	cp	1		; is it 1
	jr	z, ram_3
	cp	3		; is it 3
	jr	z, ram_4

ram_0:
	ld	a, (X7FFD)	; last value written
	and	11111000b	; page 0
	ld	(X7FFD), a	; save it
	ld	bc, 0x7ffd	; RAM paging
	out	(c), a
	jr	print_ram

ram_1:
	ld	a, (X7FFD)	; last value written
	and	11111000b	; ignore current RAM page value
	add	a, 1		; page 1
	ld	(X7FFD), a	; save it
	ld	bc, 0x7ffd	; RAM paging
	out	(c), a
	jr	print_ram

ram_3:
	ld	a, (X7FFD)	; last value written
	and	11111000b	; ignore current RAM page value
	add	a, 3		; page 3
	ld	(X7FFD), a	; save it
	ld	bc, 0x7ffd	; RAM paging
	out	(c), a
	jr	print_ram

ram_4:
	ld	a, (X7FFD)	; last value written
	and	11111000b	; ignore current RAM page value
	add	a, 4		; page 4
	ld	(X7FFD), a	; save it
	ld	bc, 0x7ffd	; RAM paging
	out	(c), a

print_ram:
	and	00000111b	; mask off page
	add	a, 48		; convert to ASCII
	ld	hl, 0x0b17	; PRINT AT 13,11
	ld	(s_posn), hl
	ld	hl, 0x48ca
	ld	(df_cc), hl
	jp	print_out	; update menu

copy_from_rom_bank_3:
	ld	a, (X1FFD)
	set	2, a		; set high ROM bit
	ld	bc, 0x1ffd
	out	(c), a
	ld	b, 0x7f
	ld	a, (X7FFD)
	set	4, a		; set low ROM bit
	out	(c), a
	jr	do_copy

copy_from_rom_bank_2:
	ld	a, (X1FFD)
	set	2, a		; set high ROM bit
	ld	bc, 0x1ffd
	out	(c), a
	ld	b, 0x7f
	ld	a, (X7FFD)
	res	4, a		; clear low ROM bit
	out	(c), a
	jr	do_copy

copy_from_rom_bank_1:
	ld	a, (X1FFD)
	res	2, a		; clear high ROM bit
	ld	bc, 0x1ffd
	out	(c), a
	ld	b, 0x7f
	ld	a, (X7FFD)
	set	4, a		; set low ROM bit
	out	(c), a
	jr	do_copy

copy_from_rom_bank_0:
	ld	a, (X1FFD)
	res	2, a		; clear high ROM bit
	ld	bc, 0x1ffd
	out	(c), a
	ld	b, 0x7f
	ld	a, (X7FFD)
	res	4, a		; clear low ROM bit
	out	(c), a

do_copy:
	ld	hl, 0x0000	; source
	ld	de, 0xc000	; dest
	ld	bc, 0x4000	; length
	ldir			; copy it
	ld	a, (X1FFD)	; restore previous paging
	ld	bc, 0x1ffd
	out	(c), a
	ld	b, 0x7f
	ld	a, (X7FFD)
	out	(c), a
	ret

; routine nicked from Geoff Wearmouth's Utility 3
; which was in turn nicked from the Spectrum ROM

key1:
	call	kscan
	ld	hl, table

kloop:
	cp	(hl)
	jr	z, found
	inc	l
	jr	z, key1
	jr	kloop

found:
	ld	a, l
	cpl
	ret

kscan:
	ld	de, 0xff2f
	ld	bc, 0xfefe

kline:
	in	a, (c) 
	cpl
	and	0x1f
	jr	z, kdone
	inc	d
	jr	nz, kscan
	ld	h,a
	ld	a,e

kbits:
	sub	8
	srl	h 
	jr	nc, kbits
	jr	nz, kscan
	ld	d, a

kdone:
	dec	e
	rlc	b
	jr	c, kline
	ld	a, d
	inc	d
	jr	z, kscan
	ret

; routine nicked from the Spectrum ROM

print_out:
	ld	bc, (s_posn)
	ld	hl, (df_cc)
	cp	' '
	jr	c, p_enter
	call	po_char
	jr	po_store
	
p_enter:
	ld	c, 33
	dec	b
	ld	a, b
	push	bc
	ld	b, a
	ld	a, 24
	sub	b
	ld	d, a
	rrca
	rrca
	rrca
	and	11100000b
	ld	l, a
	ld	a, d
	and	00011000b
	or	01000000b
	ld	h, a
	pop	bc
	ld	a, 33
	sub	c
	ld	e, a
	ld	d, 0
	add	hl, de

po_store:
	ld	(s_posn), bc
	ld	(df_cc), hl
	ret

po_char:
	push	bc
	ld	bc, chars - 0x100

po_char_2:
	ex	de, hl

po_char_3:
	ld	h, 0
	ld	l, a
	add	hl, hl
	add	hl, hl
	add	hl, hl
	add	hl, bc
	pop	bc
	ex	de, hl

pr_all:
	ld	a, c
	dec	a
	ld	a, 33
	jr	nz, pr_all_1
	dec	b
	ld	c, a

pr_all_1:
	cp	c
	push	bc
	push	hl
	ld	b, 0

pr_all_2:
	rra
	rra
	sbc	a, a
	ld	c, a
	ld	a, 8
	and	a

pr_all_3:
	ex	de, hl

pr_all_4:
	ex	af, af'
	ld	a, (de)
	and	b
	xor	(hl)
	xor	c
	ld	(de), a
	ex	af, af'
	jr	c, pr_all_6
	inc	d

pr_all_5:
	inc	hl
	dec	a
	jr	nz, pr_all_4
	ex	de, hl
	dec	h
	pop	hl
	pop	bc
	dec	c
	inc	hl
	ret

pr_all_6:
	ex	af, af'
	ld	a, 32
	add	a, e
	ld	e, a
	ex	af, af'
	jr	pr_all_5

po_msg:
	ld	h, 0
	call	po_search

po_each:
	ld	a, (de)
	and	01111111b
	call	po_save
	ld	a, (de)
	inc	de
	add	a, a
	jr	nc, po_each
	ret

po_save:
	push	de
	call	print_out
	pop	de
	ret

po_search:
	push	af
	ex	de, hl
	inc	a

po_step:
	bit	7, (hl)
	inc	hl
	jr	z, po_step
	dec	a
	jr	nz, po_step
	ex	de, hl
	pop	af
	cp	' '
	ret	c
	ld	a, (de)
	sub	'A'
	ret

; LOAD a 16K ROM file (with header) from TAPE to 0xc000

load_rom_bank_from_tape:

; copy the routine to non-contended RAM first

	ld	hl, sa_ld_ret
	ld	de, 0x8000
	ld	bc, 0x00FE
	ldir			; copy routine to non-contended ram	

; original routine resumes

	scf
	xor	a
	ld	de, 17
	ld	ix, 0xc000
	call	0x8015	;ld_bytes
	scf
	ld	a, 0xff
	ld	de, 0x4000
	ld	ix, 0xc000

	jp	0x8015	;ld_bytes

; routines nicked from the Spectrum ROM

sa_ld_ret:
	push	af
	xor	a
	and	00111000b
	rrca
	rrca
	rrca
	out	(0xfe), a
	ld	a, 127
	in	a, (0xfe)
	rra
	jr	c, sa_ld_end

report_da:
	rst	08h
	defb	0x0c
	
sa_ld_end:
	jp ae_ld_ck      ; ae-> substituted with
                         ; POP AF
	                 ; ret

ld_bytes:
	inc	d
	ex	af, af'
	dec	d
	di
	ld	a, 15
	out	(0xfe), a
	ld	hl, 0x8000	; sa_ld_ret
	push	hl
	in	a, (0xfe)
	rra
	and	00100000b
	or	7		; white
	ld	c, a
	cp	a

ld_break:
	ret	nz

ld_start:
	call	ld_edge_1
	jr	nc, ld_break
	ld	hl, 0x0415

ld_wait:
	djnz	ld_wait
	dec	hl
	ld	a, h
	or	l
	jr	nz, ld_wait
	call	ld_edge_2
	jr	nc, ld_break

ld_leader:
	ld	b, 156
	call	ld_edge_2
	jr	nc, ld_break
	ld	a, 198
	cp	b
	jr	nc, ld_start
	inc	h
	jr	nz, ld_leader

ld_sync:
	ld	b, 201
	call	ld_edge_1
	jr	nc, ld_break
	ld	a, b
	cp	212
	jr	nc, ld_sync
	call	ld_edge_1
	ret	nc
	ld	a, c
	xor	00000000b	
	ld	c, a
	ld	h, 0
	ld	b, 176
	jr	ld_marker

ld_loop:
	ex	af, af'
	jr	nz, ld_flag
	jr	nc, ld_verify
	ld	(ix + 0x00), l
	jr	ld_next

ld_flag:
	rl	c
	xor	l
	ret	nz
	ld	a, c
	rra
	ld	c, a
	inc	de
	jr	ld_dec

ld_verify:
	ld	a, (ix + 0x00)
	xor	l
	ret	nz

ld_next:
	inc	ix

ld_dec:
	dec	de
	ex	af, af'
	ld	b, 178

ld_marker:
	ld	l, 00000001b

ld_8_bits:
	call	0x80A2	;ld_edge_2
	ret	nc
	ld	a, 203
	cp	b
	rl	l
	ld	b, 176
	jp	nc, 0x8089	;ld_8_bits
	ld	a, h
	xor	l
	ld	h, a
	ld	a, d
	or	e
	jr	nz, ld_loop 
	ld	a, h
	cp	1
	ret

ld_edge_2:
	call	0x80A6	;ld_edge_1
	ret	nc

ld_edge_1:
	ld	a, 22

ld_delay:
	dec	a
	jr	nz, ld_delay
	and	a

ld_sample:
	inc	b
	ret	z
	ld	a, 127
	in	a, (0xfe)
	rra
	ret	nc
	xor	c
	and	00100000b
	jr	z, ld_sample
	ld	a, c
	cpl
	ld	c, a
	and	00000111b
	or	00001000b
	out	(0xfe), a
	scf
	ret


ae_ld_ck:                       ; ** "ae" labels are added by ref to check
        ld a,d                  ; ** if 16384 bytes loaded by loader
        or e                    ; check if DE reached zero
        JR NZ,ae_fail
        
	ld 	a, 0x04         ; 16k rom data read.
	out	(0xfe), a       ; set border to green
        jp ae_exit

ae_fail:
        ld	a, 2		; obviously we couldn't load 16384 bytes,
	out	(0xfe), a	; set a red border as a warning

ae_exit:                        ; originally  sa_ld_end
	pop	af
	ret

; variables

X1FFD:
	defb	0		; value of last port write to 0x1ffd

; port 0x1ffd
;
; d0   all-RAM select
; d1   low all-RAM scheme 
; d2   high ROM bit / all-RAM scheme 
; d3   disk motor
; d4   printer strobe
; d5-7 not used

X7FFD:
	defb	3		; value of last port write to 0x7ffd

; port 0x7ffd
;
; d0-2 RAM page
; d3   VIDEO page
; d4   low ROM bit 
; d5   48 lock
; d6-7 not used

s_posn:
	defb	33		; screen position
	defb	24
df_cc:
	defw	0x4000		; display file address
p_flag:
	defw	0

menu:
	defm	0x80
	defm	"+3/2A FLASH ROM BOARD PROGRAMMER", 13
	defm	13
	defm	"0 - COPY FROM ROM BANK 0", 13
	defm	"1 - COPY FROM ROM BANK 1", 13
	defm	"2 - COPY FROM ROM BANK 2", 13
	defm	"3 - COPY FROM ROM BANK 3", 13
	defm	"4 - CHANGE RAM BANK", 13
	defm	"5 - CHANGE ROM BANK ", 13
	defm	"6 - LOAD ROM IMAGE FROM TAPE  ", 13
	defm	"7 - WIPE ROM BANK  ", 13
	defm	"8 - FLASH ROM BANK   ", 13
	defm	"9 - BOOT ACTIVE ROM ", 13
	defm	13
	defm	"RAM BANK: 0     ROM BANK: ", "0", + 0x80

; font nicked from the Spectrum ROM

chars:
	defb	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ; SPACE
	defb	0x00, 0x10, 0x10, 0x10, 0x10, 0x00, 0x10, 0x00
	defb	0x00, 0x24, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00
	defb	0x00, 0x24, 0x7e, 0x24, 0x24, 0x7e, 0x24, 0x00
	defb	0x00, 0x08, 0x3e, 0x28, 0x3e, 0x0a, 0x3e, 0x08
	defb	0x00, 0x62, 0x64, 0x08, 0x10, 0x26, 0x46, 0x00
	defb	0x00, 0x10, 0x28, 0x10, 0x2a, 0x44, 0x3a, 0x00
	defb	0x00, 0x08, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00
	defb	0x00, 0x04, 0x08, 0x08, 0x08, 0x08, 0x04, 0x00
	defb	0x00, 0x20, 0x10, 0x10, 0x10, 0x10, 0x20, 0x00
	defb	0x00, 0x00, 0x14, 0x08, 0x3e, 0x08, 0x14, 0x00
	defb	0x00, 0x00, 0x08, 0x08, 0x3e, 0x08, 0x08, 0x00
	defb	0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x08, 0x10
	defb	0x00, 0x00, 0x00, 0x00, 0x3e, 0x00, 0x00, 0x00
	defb	0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00
	defb	0x00, 0x00, 0x02, 0x04, 0x08, 0x10, 0x20, 0x00
	defb	0x00, 0x3c, 0x46, 0x4a, 0x52, 0x62, 0x3c, 0x00
	defb	0x00, 0x18, 0x28, 0x08, 0x08, 0x08, 0x3e, 0x00
	defb	0x00, 0x3c, 0x42, 0x02, 0x3c, 0x40, 0x7e, 0x00
	defb	0x00, 0x3c, 0x42, 0x0c, 0x02, 0x42, 0x3c, 0x00
	defb	0x00, 0x08, 0x18, 0x28, 0x48, 0x7e, 0x08, 0x00
	defb	0x00, 0x7e, 0x40, 0x7c, 0x02, 0x42, 0x3c, 0x00
	defb	0x00, 0x3c, 0x40, 0x7c, 0x42, 0x42, 0x3c, 0x00
	defb	0x00, 0x7e, 0x02, 0x04, 0x08, 0x10, 0x10, 0x00
	defb	0x00, 0x3c, 0x42, 0x3c, 0x42, 0x42, 0x3c, 0x00
	defb	0x00, 0x3c, 0x42, 0x42, 0x3e, 0x02, 0x3c, 0x00
	defb	0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x10, 0x00
	defb	0x00, 0x00, 0x10, 0x00, 0x00, 0x10, 0x10, 0x20
	defb	0x00, 0x00, 0x04, 0x08, 0x10, 0x08, 0x04, 0x00
	defb	0x00, 0x00, 0x00, 0x3e, 0x00, 0x3e, 0x00, 0x00
	defb	0x00, 0x00, 0x10, 0x08, 0x04, 0x08, 0x10, 0x00
	defb	0x00, 0x3c, 0x42, 0x04, 0x08, 0x00, 0x08, 0x00
	defb	0x00, 0x3c, 0x4a, 0x56, 0x5e, 0x40, 0x3c, 0x00
	defb	0x00, 0x3c, 0x42, 0x42, 0x7e, 0x42, 0x42, 0x00
	defb	0x00, 0x7c, 0x42, 0x7c, 0x42, 0x42, 0x7c, 0x00
	defb	0x00, 0x3c, 0x42, 0x40, 0x40, 0x42, 0x3c, 0x00
	defb	0x00, 0x78, 0x44, 0x42, 0x42, 0x44, 0x78, 0x00
	defb	0x00, 0x7e, 0x40, 0x7c, 0x40, 0x40, 0x7e, 0x00
	defb	0x00, 0x7e, 0x40, 0x7c, 0x40, 0x40, 0x40, 0x00
	defb	0x00, 0x3c, 0x42, 0x40, 0x4e, 0x42, 0x3c, 0x00
	defb	0x00, 0x42, 0x42, 0x7e, 0x42, 0x42, 0x42, 0x00
	defb	0x00, 0x3e, 0x08, 0x08, 0x08, 0x08, 0x3e, 0x00
	defb	0x00, 0x02, 0x02, 0x02, 0x42, 0x42, 0x3c, 0x00
	defb	0x00, 0x44, 0x48, 0x70, 0x48, 0x44, 0x42, 0x00
	defb	0x00, 0x40, 0x40, 0x40, 0x40, 0x40, 0x7e, 0x00
	defb	0x00, 0x42, 0x66, 0x5a, 0x42, 0x42, 0x42, 0x00
	defb	0x00, 0x42, 0x62, 0x52, 0x4a, 0x46, 0x42, 0x00
	defb	0x00, 0x3c, 0x42, 0x42, 0x42, 0x42, 0x3c, 0x00
	defb	0x00, 0x7c, 0x42, 0x42, 0x7c, 0x40, 0x40, 0x00
	defb	0x00, 0x3c, 0x42, 0x42, 0x52, 0x4a, 0x3c, 0x00
	defb	0x00, 0x7c, 0x42, 0x42, 0x7c, 0x44, 0x42, 0x00
	defb	0x00, 0x3c, 0x40, 0x3c, 0x02, 0x42, 0x3c, 0x00
	defb	0x00, 0xfe, 0x10, 0x10, 0x10, 0x10, 0x10, 0x00
	defb	0x00, 0x42, 0x42, 0x42, 0x42, 0x42, 0x3c, 0x00
	defb	0x00, 0x42, 0x42, 0x42, 0x42, 0x24, 0x18, 0x00
	defb	0x00, 0x42, 0x42, 0x42, 0x42, 0x5a, 0x24, 0x00
	defb	0x00, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0x00
	defb	0x00, 0x82, 0x44, 0x28, 0x10, 0x10, 0x10, 0x00
	defb	0x00, 0x7e, 0x04, 0x08, 0x10, 0x20, 0x7e, 0x00 ; 'Z'

